/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2004-2005 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without 
any warranty. CUSTOMER retains the final decision relative to the total design 
and functionality of the end product. FREESCALE neither guarantees nor will be 
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING, 
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR 
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE OR ADVISE SUPPLIED TO THE PROJECT
BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE SERVICES. IN NO EVENT
SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THIS AGREEMENT.

CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands or
actions by anyone on account of any damage, or injury, whether commercial, 
contractual, or tortuous, rising directly or indirectly as a result of an advise
or assistance supplied CUSTOMER in connection with product, services or goods 
supplied under this Agreement.
********************************************************************************
* File      convert.c
* Owner     r62780
* Version   1.1
* Date      Apr-15-2010
* Classification   General Business Information
* Brief     Conversions
********************************************************************************
* Detailed Description: The file contains conversions

********************************************************************************
Revision History:
Version  Date         Author  Description of Changes
1.0      Apr-13-2010  R62780  Initial version
1.1      Apr-15-2010  R62780  Tabelators substituded by empty spaces
*******************************************************************************/
#include "convert.h"                             // derivative information
/*******************************************************************************
* Global functions
*******************************************************************************/
/*******************************************************************************
Function Name  : ConvertPpageOff2Global
Engineer       : r62780
Date           : Apr-15-2010
Parameters     : ULONG addr
Modifies       : NONE
Returns        : NONE
Notes          : Convert data address format from 0xPPAGE~offset to the global
                 address format 0xGPAGE~offset
               Conversion flow:
                addr       - function: 0000 0000 pppp pppp ffff ffff ffff ffff 
                                                 1111 1198 7654 3210 7654 3210 
              		||                             5432 10                 
              		\/                                                     
                Global addr          :  1 PPAGE Register [7:0]   address [13:0]
                                                                               
                                     : 0000 0000 01pp pppp ppff ffff ffff ffff 
                                                   76 5432 1011 1198 7654 3210 
              		                                           32 10       
Issues         : NONE
*******************************************************************************/
ULONG  ConvertPpageOff2Global(ULONG addr)
{
  addr = ((addr>>2) & 0x003FC000UL) | (addr & 0x00003FFFUL) | 0x00400000;
  return(addr);
}
/*******************************************************************************
Function Name  : ConvertPpageOff2Global
Engineer       : r62780
Date           : Apr-15-2010
Parameters     : ULONG addr
Modifies       : NONE
Returns        : NONE
Notes          : Convert data address format from global address format 
                 0xGPAGE~offset to the format 0xPPAGE~offset
               
               It is made because compiler uses global addressing (GLY, GLAB) 
               for far data access. 
               
               Conversion flow:
                Global addr          :  1 PPAGE Register [7:0]   address [13:0]
                                                                               
                  ||                 : 0000 0000 01pp pppp ppff ffff ffff ffff 
                  \/                               76 5432 1011 1198 7654 3210 
              		                                         32 10         
                addr       - function: 0000 0000 pppp pppp ffff ffff ffff ffff 
                                                 1111 1198 7654 3210 7654 3210 
              		                               5432 10                 
Issues         : NONE
*******************************************************************************/
ULONG  ConvertGlobal2PpageOff(ULONG addr)
{
  addr = ((addr<<2) & 0x00FF0000UL) | (addr & 0x00003FFFUL);
  return(addr);
}
//==============================================================================
