/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2004-2005 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without 
any warranty. CUSTOMER retains the final decision relative to the total design 
and functionality of the end product. FREESCALE neither guarantees nor will be 
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING, 
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR 
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE OR ADVISE SUPPLIED TO THE PROJECT
BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE SERVICES. IN NO EVENT
SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THIS AGREEMENT.

CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands or
actions by anyone on account of any damage, or injury, whether commercial, 
contractual, or tortuous, rising directly or indirectly as a result of an advise
or assistance supplied CUSTOMER in connection with product, services or goods 
supplied under this Agreement.
********************************************************************************
* File      main.c
* Owner     r62780
* Version   1.1
* Date      Apr-15-2010
* Classification   General Business Information
* Brief     FLASH E/W
********************************************************************************
* Detailed Description: Flash routines for S12XD family devices

********************************************************************************
Revision History:
Version  Date         Author  Description of Changes
1.0      Apr-13-2010  R62780  Initial version
1.1      Apr-15-2010  R62780  Tabelators substituded by empty spaces
*******************************************************************************/
/*******************************************************************************
* External objects
*******************************************************************************/
#include "flash.h"
#include "convert.h"
/*******************************************************************************
* Global variables
*******************************************************************************/
// none
/*******************************************************************************
* Constants and macros
*******************************************************************************/
#define PUIF  unsigned int *far

/*macro which stores CCR and disables I-bit maskable interupts*/
#define GetCCRdisI() { asm TPA; asm STAA ccrBackup; asm SEI; }

/*macro which restores CCR to the status before after GetCCRdisI() was called*/
#define restoreCCR()  { asm LDAA ccrBackup; asm TAP;  }
/*******************************************************************************
* Local types
*******************************************************************************/
//none
/*******************************************************************************
* Local function prototypes
*******************************************************************************/
static UBYTE PerformWriteWord(ULONG addr, UWORD data);
static UBYTE Flash_Busy(void);
/*******************************************************************************
* Local variables
*******************************************************************************/
static UBYTE ccrBackup;             // stores CCR after GetCCRdisI() was called

/*
  variable stores specific code in RAM which is used for waiting till flash 
  activity is finished. The Code Warrior Startup function ensures the variable
  is copyied after MCU start from the flash to RAM
*/
static UBYTE WaitForFlash[] =
{
 0xC6, 0x80,                        // LDAB #$80     // command buffer empty = 0
 0x7B, 0x01, 0x05,                  // STAB _FSTAT
 
 0xC7,                              // CLRB          // clear return value
 
 0x1E, 0x01, 0x05, 0x20, 0x05,      // BRSET _FSTAT,#32,*+10 
 0x1F, 0x01, 0x05, 0x10, 0x03,      // BRCLR _FSTAT,#16,*+8 
 0xC6, 0x08,                        // LDAB  #8             ;ERRNOTAVAIL
 0x3D,                              // RTS 
 0x1F, 0x01, 0x05, 0x40, 0xFB,      // BRCLR _FSTAT,#64,*+0 
 0x3D                               // RTS
};
/*******************************************************************************
* Local functions
*******************************************************************************/
/*******************************************************************************
Function Name  : PerformWriteWord
Engineer       : r62780
Date           : Apr-15-2010
Parameters     : UWORD addr      
                 UWORD data
Modifies       : NONE
Returns        : NONE
Notes          : Writes word and waits while flash is running
Issues         : NONE
*******************************************************************************/
static UBYTE PerformWriteWord(ULONG addr, UWORD data)
{
 UBYTE err;
                              
 GetCCRdisI();                                  // Save CCR, disable interrupts
 FSTAT = 48;                                    // Clear PVIOL & ACERR flag
 if(FSTAT_CBEIF == 0)                           // Is command buffer full ?  
  {restoreCCR();
   return ERRBUSY;
  } 
 *(UWORD *far) addr = data;                     // address and program data
 FCMD = 0x20;                                   // Word program command
 asm JSR WaitForFlash; asm STAB err;
 restoreCCR();                                  // Restore CCR
 if(err) return err;                            // if error return error code
 if(*(UWORD *far) addr != data) return ERRVALUE;// Check written data
 return ERROK;
}
/*******************************************************************************
Function Name  : Flash_Busy
Engineer       : r62780
Date           : Apr-15-2010
Parameters     : NONE
Modifies       : NONE
Returns        : NONE
Notes          : Gets status of the flash
Issues         : NONE
*******************************************************************************/
static UBYTE Flash_Busy(void)
{
  return((!FSTAT_CBEIF) || (!FSTAT_CCIF));
}
/*******************************************************************************
* Global functions
*******************************************************************************/
/*******************************************************************************
Function Name  : Flash_WriteWordLOCAL
Engineer       : r62780
Date           : Apr-15-2010
Parameters     : UWORD addr
                 UWORD data
Modifies       : CCR
Returns        : NONE
Notes          : Write data to non paged memory space
                 The function accept only unpaged flash 4000~BFFF and C000~FFFF
Issues         : I-bit maskable interrupts are (and must be) disabled while
                 Flash is running
*******************************************************************************/
UBYTE Flash_WriteWordLOCAL(UWORD addr,UWORD data)
{
  UBYTE err;

  if((addr<0x4000) || (addr>0x7FFF))
    {if(addr<0xC000) return(ERRRANGE);  // Check range of address
    }
  if(addr & 1) return ERRNOTAVAIL;      // Aligned address ?

  GetCCRdisI();                         // Save CCR, disable interrupts

  FSTAT = 48;                           // Clear PVIOL & ACERR flag

  if(FSTAT_CBEIF == 0)                  // Is command buffer full ?
   { restoreCCR();
     return ERRBUSY;
   } 
  *(UWORD *) addr = data;               // Array address and program data XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  FCMD = 0x20;                          // Word program command
  asm JSR WaitForFlash; 
  asm STAB err;                         // store return val from reg. B to err
  restoreCCR();                         // Restore CCR
  if (err) return err;                  // if error return error code
  if (*(UWORD *) addr != data) return ERRVALUE;   // Check written data
  return ERROK;
}
/*******************************************************************************
Function Name  : Flash_WriteWordPPAGE
Engineer       : r62780
Date           : Apr-15-2010
Parameters     : UWORD addr
                 UWORD data
Modifies       : NONE
Returns        : NONE
Notes          : Writes word to flash address inserted in PPAGE~OFFSET format 
                 0xPPOOOO
Issues         : NONE
*******************************************************************************/
UBYTE Flash_WriteWordPPAGE(ULONG addr,UWORD data)
{
  UBYTE err;
  addr = ConvertPpageOff2Global(addr);
  err  = Flash_WriteWordGLOBAL(addr,data);
  return(err);                         /* Return error */
}
/*******************************************************************************
Function Name  : Flash_WriteWordGLOBAL
Engineer       : r62780
Date           : Apr-15-2010
Parameters     : UWORD addr
                 UWORD data
Modifies       : NONE
Returns        : NONE
Notes          : Writes word to flash address inserted in GPAGE~OFFSET format
Issues         : NONE
*******************************************************************************/
UBYTE Flash_WriteWordGLOBAL(ULONG addr,UWORD data)
{
  UBYTE err;

  // Check range of address
  if((addr<0x780000) || (addr>0x7FFF00)) return(ERRRANGE);
  if(addr & 1) return ERRNOTAVAIL;                // Aligned address ?
  err = PerformWriteWord(addr, data);             // Write new data to Flash
  return(err);                                    // Return error 
}
/*******************************************************************************
Function Name  : Flash_EraseSectorLOCAL
Engineer       : r62780
Date           : Apr-15-2010
Parameters     : UWORD addr
                 UWORD data
Modifies       : NONE
Returns        : NONE
Notes          : Erase sector in non paged memory space
                 The function accept only unpaged flash 4000~BFFF and C000~FFFF
Issues         : NONE
*******************************************************************************/
UBYTE Flash_EraseSectorLOCAL(UWORD addr)
{
  UBYTE err;

  if((addr<0x4000) || (addr>0x7FFF))
    {if(addr<0xC000) return(ERRRANGE);         // Check range of address
    }
  GetCCRdisI();                                // Save CCR, disable interrupts
  FSTAT = 48;                                  // Clear PVIOL & ACERR flag
 
  if (FSTAT_CBEIF == 0)                        // Is command buffer full ?
   { restoreCCR();
     return ERRBUSY;
   } 
  *(UWORD *) addr = 0xFFFF;                    // Write any word to FLASH buffer
  FCMD = 0x40;                                 // Erase Sector commamd */
  asm JSR WaitForFlash; asm STAB err;
  restoreCCR();                                // Restore CCR
  
  // Return error code if previous operation finished not correctly
  if (err)   return err; 

  return ERROK;
}
/*******************************************************************************
Function Name  : Flash_EraseSectorPPAGE
Engineer       : r62780
Date           : Apr-15-2010
Parameters     : UWORD addr
Modifies       : NONE
Returns        : NONE
Notes          : Erase sector in PPAGE paged memory space (PPAGE~OFFSET) 0xPPOOOO
Issues         : NONE
*******************************************************************************/
UBYTE Flash_EraseSectorPPAGE(ULONG addr)
{
  UBYTE err;
  addr = ConvertPpageOff2Global(addr);
  err  = Flash_EraseSectorGLOBAL(addr);
  return(err);                         /* Return error */
}
/*******************************************************************************
Function Name  : Flash_EraseSectorGLOBAL
Engineer       : r62780
Date           : Apr-15-2010
Parameters     : UWORD addr
Modifies       : NONE
Returns        : NONE
Notes          : Erase sector in GPAGE paged mem space (GPAGE~OFFSET) 0xGGOOOO
Issues         : NONE
*******************************************************************************/
UBYTE Flash_EraseSectorGLOBAL(ULONG addr)
{
 UBYTE err;

 if((addr<0x780000)||(addr>0x7FFF00))           // Check range of address
                             return(ERRRANGE);  
 GetCCRdisI();                                  // Save CCR, disable interrupts
 FSTAT = 48;                                    // Clear PVIOL & ACERR flag

  if (FSTAT_CBEIF == 0)                         // Is command buffer full ?
   { restoreCCR();
     return ERRBUSY;
   } 
 *(UWORD *far) addr = 0xFFFF;                   // Write a word to FLASH buffer
 FCMD = 0x40;                                   // Erase Sector commamd 
 asm JSR WaitForFlash; asm STAB err;
 restoreCCR();                                  // Restore CCR
 
 // Return error code if previous operation finished not correctly
 if (err)   return err;                 
 return ERROK;
}
/*******************************************************************************
Function Name  : Flash_ErasePagePPAGE
Engineer       : r62780
Date           : Apr-15-2010
Parameters     : UWORD addr
Modifies       : NONE
Returns        : NONE
Notes          : Erase page in PPAGE paged memory space (PPAGE~OFFSET) 0xPPOOOO
Issues         : NONE
*******************************************************************************/
UBYTE Flash_ErasePagePPAGE(UBYTE ppage)
{
  UBYTE i;
  UBYTE err;
  if(ppage<0xE0) return(ERRRANGE);               // Check range of address
 
  for(i=0; i<(FLASHPAGESIZE/FLASHSECTORSIZE); i++) 
    {
      err = Flash_EraseSectorPPAGE(((((ULONG)ppage)<<16) | 0x8000) \
            + i*FLASHSECTORSIZE);
      
      // Return error code if previous operation finished not correctly
      if (err)   return err;   
    }
 return ERROK;
}
/*******************************************************************************
Function Name  : Flash_ErasePageGLOBAL
Engineer       : r62780
Date           : Apr-15-2010
Parameters     : UWORD addr
Modifies       : NONE
Returns        : NONE
Notes          : Erase page in GPAGE paged memory space (GPAGE~OFFSET) 0xGGOOOO
Issues         : NONE
*******************************************************************************/
UBYTE Flash_ErasePageGLOBAL(UBYTE gpage)
{
  UBYTE i;
  UBYTE err;
  if((gpage<0x78)||(gpage>0x7F)) return(ERRRANGE);// Check range of address
 
  for( i=0; i<(FLASHGLOBALPAGESIZE / FLASHSECTORSIZE); i++) 
    {
      err = Flash_EraseSectorGLOBAL((((ULONG)gpage)<<16) \
            + i*FLASHSECTORSIZE);
      
      // Return error code if previous operation finished not correctly
      if (err)   return err;    
    }
 return ERROK;
}
/*******************************************************************************
Function Name  : Flash_Init
Engineer       : r62780
Date           : Apr-15-2010
Parameters     : UWORD addr
Modifies       : NONE
Returns        : NONE
Notes          : Initialize Flash NVM for HCS12 by programming FCLKDIV based on 
                 passed oscillator frequency
Issues         : NONE
*******************************************************************************/
UBYTE Flash_Init(ULONG oscclk, ULONG tbus) // [kHz,ns]
{
  unsigned long fclkVal;
  unsigned long temp;

  // tbus > 1us (1000ns) (page 35 S12FTS256KV2.pdf )
  if(tbus > 1000L) return PROGRAMERASEIMPOSSIBLE; 
  
  temp = (oscclk*1000) % 8;
  fclkVal = (oscclk  * (5000UL+tbus)) / 1000000UL;

  if (oscclk >= 12800UL)
    { fclkVal = fclkVal / 8;
      if(temp) { fclkVal = fclkVal - 1; }       // oscclk % 8 is not an integer
      FCLKDIV = (unsigned char) (fclkVal | 0x40);
    }
  else
    { if(temp) { fclkVal = (fclkVal-1); }       // oscclk % 8 is not an integer
      FCLKDIV = (unsigned char) fclkVal;
    }  

  // test computed parameters
  if (oscclk>=12800L) {temp = 8UL*1000000UL*(1+FCLKDIV&0x3F) / oscclk + tbus;}
  else                {temp = 1000000UL*(1+FCLKDIV&0x3F) / oscclk + tbus;    }  
  temp = temp / 1000;
  if(temp < 5UL) return PROGRAMERASEIMPOSSIBLE; 
  FSTAT= 0x30;
}
/*******************************************************************************
If you have any questions then: The Answer to the Ultimate Question of Life, 
the Universe, and Everything is:.......... 42. 
(Douglas Adams - The Hitchhiker's Guide to the Galaxy)
*******************************************************************************/
