/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2004-2005 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without 
any warranty. CUSTOMER retains the final decision relative to the total design 
and functionality of the end product. FREESCALE neither guarantees nor will be 
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING, 
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR 
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE OR ADVISE SUPPLIED TO THE PROJECT
BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE SERVICES. IN NO EVENT
SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THIS AGREEMENT.

CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands or
actions by anyone on account of any damage, or injury, whether commercial, 
contractual, or tortuous, rising directly or indirectly as a result of an advise
or assistance supplied CUSTOMER in connection with product, services or goods 
supplied under this Agreement.
********************************************************************************
* File      main.c
* Owner     r62780
* Version   1.1
* Date      Apr-15-2010
* Classification   General Business Information
* Brief     FLASH E/W
********************************************************************************
* Detailed Description: Flash routines for S12XD family devices
********************************************************************************
Revision History:
Version  Date         Author  Description of Changes
1.0      Apr-13-2010  R62780  Initial version
1.1      Apr-15-2010  R62780  Tabelators substituded by empty spaces
*******************************************************************************/
#ifndef _FLASH_H /* Prevent duplicated includes */
#define _FLASH_H

#include "map.h"

/* Specification of the MC9S12XDP512 FLASH memory */
#define PROG                0x20          /* Word Program commnad  */
#define ERASE               0x40          /* Sector Erase command  */
#define FLASHPAGESIZE       0x4000        /* 16kB bytes PAGE       */
#define FLASHSECTORSIZE     0x400         /* 1024 bytes per sector */
#define FLASHGLOBALPAGESIZE 0x10000       /* 64kB per global page  */

/* Functions from Flash_.c */
extern UBYTE Flash_WriteWordLOCAL(UWORD Addr,UWORD Data);
extern UBYTE Flash_EraseSectorLOCAL(UWORD Addr);
extern UBYTE Flash_WriteWordPPAGE(ULONG Addr,UWORD Data);
extern UBYTE Flash_WriteWordGLOBAL(ULONG Addr,UWORD Data);
extern UBYTE Flash_EraseSectorPPAGE(ULONG Addr);
extern UBYTE Flash_EraseSectorGLOBAL(ULONG Addr);
extern UBYTE Flash_ErasePagePPAGE(UBYTE ppage);
extern UBYTE Flash_ErasePageGLOBAL(UBYTE gpage);
extern UBYTE Flash_Init(ULONG oscclk, ULONG tbus); // [kHz,ns]


/* Error codes */     
#define ERROK                    0
#define FLASHODDACCESS           1
#define FLASHNOTERASED           2
#define ACCESSERROR              3
#define PROTECTIONERROR          4
#define NOTSTARTOFSECTOR         5
#define PROGRAMERASEIMPOSSIBLE   6
#define ERRRANGE                 7
#define ERRNOTAVAIL              8
#define ERRBUSY                  9
#define ERRVALUE                 10

#endif /*_FLASH_H*/


