/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2004-2005 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without 
any warranty. CUSTOMER retains the final decision relative to the total design 
and functionality of the end product. FREESCALE neither guarantees nor will be 
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING, 
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR 
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE OR ADVISE SUPPLIED TO THE PROJECT
BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE SERVICES. IN NO EVENT
SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THIS AGREEMENT.

CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands or
actions by anyone on account of any damage, or injury, whether commercial, 
contractual, or tortuous, rising directly or indirectly as a result of an advise
or assistance supplied CUSTOMER in connection with product, services or goods 
supplied under this Agreement.
********************************************************************************
* File      main.c
* Owner     r62780
* Version   1.1
* Date      Apr-15-2010
* Classification   General Business Information
* Brief     FLASH E/W
********************************************************************************
* Detailed Description:

* - the SW software demonstrates FLASH programming as a part of application
* - the SW uses adjusted routines from AN2720
* - the SW writes a few words into FLASH locations
    Pages FE and E0~EF may be used for data storage. Other pages are used for 
    program storage (see PRM file)
     
* - tested on: HCS12X STARTER KIT
*   - OSCCLK = 16MHz, BUSCLK = 8MHz
*   - Reference to documentation: MC9S12XDP512V2 Rev.2.17
********************************************************************************
Revision History:
Version  Date         Author  Description of Changes
1.0      Apr-13-2010  R62780  Initial version
1.1      Apr-15-2010  R62780  Tabelators substituded by empty spaces
*******************************************************************************/

#include <hidef.h>              // common defines and macros
#pragma LINK_INFO DERIVATIVE "mc9s12xdp512"

#include "map.h"
#include "convert.h"                // coversion routines
#include "flash.h"                  // API to the assembly module
/*******************************************************************************
* Local variables
*******************************************************************************/
static UWORD  *far pui;                    // pointer to far memory
static UWORD  uix;
static UBYTE  err;
static UBYTE  m;
static ULONG  i,j;
static ULONG  addr;

//..................................
// SECURE THE MCU
//..................................
/*
 This will place constant into the S19 record which will be loaded into the MCU.
 This constant will be loaded from the address FF0F into the register FSEC 
 during the reset and the MCU is secured. 
*/
// a) secure the device without backdoor key
// const unsigned int secure @ 0xFF0E = 0xFFFC; 
// b) secure the device with backdoor key
// const unsigned int secure @ 0xFF0E = 0xFFBC; 

/*
 Note: A byte at the address 0xFFFE contain bits which can set COP during reset.
 If you use backdoor key you also have to add it into the code
 
 // backdoor key is 0x0123456
 const unsigned int BackDoorKey[2] @ 0xFF00 = {0x0123, 0x4567}; 
*/ 

/*******************************************************************************
Function Name : main
Engineer      : r62780
Date          : Apr/13/2010
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Initialize Flash module and perform some E/W
Issues        : NONE
*******************************************************************************/
void main(void) 
{
  //initialization of FLASH module
  err = Flash_Init(16000,125);              // Oscillator clock = 16 000 kHz
  // or FCLKDIV = 0x4A;                     // Bus clock = 8MHz => tbus=125ns

  /*
  erase first sector of the unpaged space 4000~7FFF
  and write it by means of local address
  */ 
  err = Flash_EraseSectorLOCAL(0x4000);       
  err = Flash_WriteWordLOCAL(0x4000, 0x55AA);
  err = Flash_EraseSectorLOCAL(0x4000);
  err = Flash_WriteWordLOCAL(0x4000, 0xC3C4);

  /*
  erase one sector of the FLASH example 
  Parameter is logical address (PPAGE~offset)
  */
  err = Flash_EraseSectorPPAGE(0xE08000);   // global address 0x780000

  /*
  write/read one word into/from the FLASH example
  Parameter is logical address (PPAGE~offset)
  */
  err = Flash_WriteWordPPAGE(0xE08000, 0x0123); // global address 0x780000
  uix = *((UWORD *far)(ConvertPpageOff2Global(0xE08000))); // read data back

  err = Flash_WriteWordPPAGE(0xE08002, 0x2345); // global address 0x780002
  uix = *((UWORD *far)(ConvertPpageOff2Global(0xE08002))); // read data back
  
  /*
  erase one sector of the FLASH example
  Parameter is logical address (PPAGE~offset)
  */
  err = Flash_EraseSectorPPAGE(0xE08000); // global address 0x780000
  
  /*
  write/read one word into/from the FLASH by means of GLOBAL page addressing
  Parameter is global address (GPAGE~offset)
  */
  err = Flash_WriteWordGLOBAL(0x780000, 0x5432); 
  uix = *((UWORD *far)0x780000);                // read data back

  err = Flash_WriteWordGLOBAL(0x780002, 0x3210);
  uix = *((UWORD *far)0x780002);                // read data back
  
  /*
    erase flash from PPAGE page E0~EF (global address 0x780000~7BFFFF
    (pages are excluded from code space - see *.prm file)
  Parameter is logical address (PPAGE~offset)
  */
  for(m=0xE0; m<=0xEF; m++) 
    {
      err = Flash_ErasePagePPAGE(m);
    }
  
  /*
    write flash from PPAGE page E0~EF 
    (pages are excluded from code space - see *.prm file)
  Parameter is logical address (PPAGE~offset)
  */
  uix=0;
  for(i=0xE00000L; i<=0xEF0000L; i+=0x10000L) // set ppage  
    {for(j=0x8000; j<0x9800; j+=0x02)         // set offset to be written
        { addr = i+j;
          err = Flash_WriteWordPPAGE(addr,uix++);
        }
    }
  
  /*
    erase flash from GLOBAL page 78~7B (it is equivalent to PPAGE = E0~EF
    Parameter is global address (GPAGE~offset)
  */
  for(m=0x78; m<=0x7B; m++)
    {
      err = Flash_ErasePageGLOBAL(m);
    }
  
  /*
    write flash from GLOBAL page 78~7B (it is equivalent to PPAGE = E0~EF
    Parameter is global address (GPAGE~offset)
  */
  uix = 0x1000;
  for(i=0x780000L; i<0x7C0000L; i+=2)        // set global address
    {
      err = Flash_WriteWordGLOBAL(i, uix++);
    }
  //----------------------------------------
 for(;;);

}
/*******************************************************************************
The Answer to the Ultimate Question of Life, the Universe, and Everything is:.......... 42. 
(Douglas Adams - The Hitchhiker's Guide to the Galaxy)
*******************************************************************************/
